@@ -1,16 +1,18 @@
 use std::hash::Hash;
 
 use polars_core::export::_boost_hash_combine;
-use polars_core::export::ahash::{self, CallHasher};
+use polars_core::export::ahash::{self};
 use polars_core::export::rayon::prelude::*;
 use polars_core::utils::NoNull;
+use polars_core::POOL;
+use polars_utils::HashSingle;
 
 use super::*;
 
 fn hash_agg<T>(ca: &ChunkedArray<T>, random_state: &ahash::RandomState) -> u64
 where
     T: PolarsIntegerType,
-    T::Native: Hash + CallHasher,
+    T::Native: Hash,
 {
     // Note that we don't use the no null branch! This can break in unexpected ways.
     // for instance with threading we split an array in n_threads, this may lead to
@@ -29,7 +31,7 @@ where
         for opt_v in arr.iter() {
             match opt_v {
                 Some(v) => {
-                    let r = T::Native::get_hash(v, random_state);
+                    let r = random_state.hash_single(v);
                     hash_agg = _boost_hash_combine(hash_agg, r);
                 }
                 None => {
@@ -41,7 +43,7 @@ where
     hash_agg
 }
 
-pub(crate) fn hash(ca: &ListChunked, build_hasher: ahash::RandomState) -> UInt64Chunked {
+pub(crate) fn hash(ca: &mut ListChunked, build_hasher: ahash::RandomState) -> UInt64Chunked {
     if !ca.inner_dtype().to_physical().is_numeric() {
         panic!(
             "Hashing a list with a non-numeric inner type not supported. Got dtype: {:?}",
@@ -52,22 +54,24 @@ pub(crate) fn hash(ca: &ListChunked, build_hasher: ahash::RandomState) -> UInt64
     // just some large prime
     let null_hash = 1969099309u64;
 
-    let out: NoNull<UInt64Chunked> = ca
-        .par_iter()
-        .map(|opt_s: Option<Series>| match opt_s {
-            None => null_hash,
-            Some(s) => {
-                let s = s.to_physical_repr();
-                if s.bit_repr_is_large() {
-                    let ca = s.bit_repr_large();
-                    hash_agg(&ca, &build_hasher)
-                } else {
-                    let ca = s.bit_repr_small();
-                    hash_agg(&ca, &build_hasher)
+    ca.set_inner_dtype(ca.inner_dtype().to_physical());
+
+    let out: NoNull<UInt64Chunked> = POOL.install(|| {
+        ca.par_iter()
+            .map(|opt_s: Option<Series>| match opt_s {
+                None => null_hash,
+                Some(s) => {
+                    if s.bit_repr_is_large() {
+                        let ca = s.bit_repr_large();
+                        hash_agg(&ca, &build_hasher)
+                    } else {
+                        let ca = s.bit_repr_small();
+                        hash_agg(&ca, &build_hasher)
+                    }
                 }
-            }
-        })
-        .collect();
+            })
+            .collect()
+    });
 
     let mut out = out.into_inner();
     out.rename(ca.name());
