@@ -15,11 +15,18 @@ impl CountAgg {
     pub(crate) fn new() -> Self {
         CountAgg { count: 0 }
     }
+    fn incr(&mut self) {
+        self.count += 1;
+    }
 }
 
 impl AggregateFn for CountAgg {
+    fn has_physical_agg(&self) -> bool {
+        false
+    }
+
     fn pre_agg(&mut self, _chunk_idx: IdxSize, _item: &mut dyn ExactSizeIterator<Item = AnyValue>) {
-        self.count += 1;
+        self.incr();
     }
 
     fn dtype(&self) -> DataType {
