@@ -23,14 +23,14 @@ use crate::operators::{DataChunk, FinalizedSink, PExecutionContext, Sink, SinkRe
 
 // This is the hash and the Index offset in the linear buffer
 #[derive(Copy, Clone)]
-struct Key {
-    hash: u64,
-    idx: IdxSize,
+pub(super) struct Key {
+    pub(super) hash: u64,
+    pub(super) idx: IdxSize,
 }
 
 impl Key {
     #[inline]
-    fn new(hash: u64, idx: IdxSize) -> Self {
+    pub(super) fn new(hash: u64, idx: IdxSize) -> Self {
         Self { hash, idx }
     }
 }
@@ -237,13 +237,13 @@ impl Sink for GenericGroupbySink {
 
         // a small buffer that holds the current key values
         // if we groupby 2 keys, this holds 2 anyvalues.
-        let mut current_keys_buf = Vec::with_capacity(self.keys.len());
+        let mut current_tuple = Vec::with_capacity(self.keys.len());
 
         for &h in &self.hashes {
             // load the keys in the buffer
-            current_keys_buf.clear();
+            current_tuple.clear();
             for key_iter in key_iters.iter_mut() {
-                unsafe { current_keys_buf.push(key_iter.next().unwrap_unchecked_release()) }
+                unsafe { current_tuple.push(key_iter.next().unwrap_unchecked_release()) }
             }
 
             let partition = hash_to_partition(h, self.pre_agg_partitions.len());
@@ -256,13 +256,13 @@ impl Sink for GenericGroupbySink {
             let entry = current_partition.raw_entry_mut().from_hash(h, |key| {
                 let idx = key.idx as usize;
                 if self.keys_series.len() > 1 {
-                    current_keys_buf.iter().enumerate().all(|(i, key)| unsafe {
+                    current_tuple.iter().enumerate().all(|(i, key)| unsafe {
                         current_key_values.get_unchecked_release(i + idx) == key
                     })
                 } else {
                     unsafe {
                         current_key_values.get_unchecked_release(idx)
-                            == current_keys_buf.get_unchecked_release(0)
+                            == current_tuple.get_unchecked_release(0)
                     }
                 }
             });
@@ -278,11 +278,11 @@ impl Sink for GenericGroupbySink {
                             NumCast::from(current_key_values.len()).unwrap_unchecked_release(),
                         )
                     };
-                    entry.insert_with_hasher(h, keys_offset, value_offset, |_| h);
+                    entry.insert(keys_offset, value_offset);
 
                     unsafe {
                         current_key_values.extend(
-                            current_keys_buf
+                            current_tuple
                                 .iter()
                                 .map(|av| av.clone().into_static().unwrap_unchecked_release()),
                         )
@@ -315,6 +315,7 @@ impl Sink for GenericGroupbySink {
 
         let other = other.as_any().downcast_ref::<Self>().unwrap();
         let n_partitions = self.pre_agg_partitions.len();
+        debug_assert_eq!(n_partitions, other.pre_agg_partitions.len());
         let n_keys = self.number_of_keys();
 
         self.pre_agg_partitions
@@ -371,11 +372,11 @@ impl Sink for GenericGroupbySink {
                                     )
                                 };
 
-                                // extend the keys buffer with the new keys from othger
+                                // extend the keys buffer with the new keys from other
                                 keys_buffer_self.extend_from_slice(keys_other);
 
                                 // insert the keys and values_offset
-                                entry.insert_with_hasher(h, key, values_offset, |_| h);
+                                entry.insert(key, values_offset);
                                 // initialize the new aggregators
                                 for agg_fn in &self.agg_fns {
                                     aggregators_self.push(agg_fn.split2())
