@@ -1,6 +1,6 @@
 //! Common Subplan Elimination
 
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, BTreeSet};
 use std::hash::{BuildHasher, Hash, Hasher};
 
 use polars_core::prelude::*;
@@ -50,6 +50,10 @@ pub(super) fn collect_trails(
             collect_trails(*input_right, lp_arena, trails, id, true)?;
         }
         Union { inputs, .. } => {
+            if inputs.len() > 200 {
+                // don't even bother with cse on this many inputs
+                return None;
+            }
             let new_trail = trails.get(id).unwrap().clone();
 
             let last_i = inputs.len() - 1;
@@ -239,12 +243,14 @@ fn lp_node_equal(a: &ALogicalPlan, b: &ALogicalPlan, expr_arena: &Arena<AExpr>)
 }
 
 /// Iterate from two leaf location upwards and find the latest matching node.
+///
+/// Returns the matching nodes
 fn longest_subgraph(
     trail_a: &Trail,
     trail_b: &Trail,
     lp_arena: &Arena<ALogicalPlan>,
     expr_arena: &Arena<AExpr>,
-) -> Option<(usize, Node, Node)> {
+) -> Option<(Node, Node, bool)> {
     if trail_a.is_empty() || trail_b.is_empty() {
         return None;
     }
@@ -252,8 +258,9 @@ fn longest_subgraph(
     let mut prev_node_b = Node(0);
     let mut is_equal;
     let mut i = 0;
+    let mut entirely_equal = trail_a.len() == trail_b.len();
 
-    // iterates from the leafs upwards
+    // iterates from the leaves upwards
     for (node_a, node_b) in trail_a.iter().rev().zip(trail_b.iter().rev()) {
         // we never include the root that splits a trail
         // e.g. don't want to cache the join/union, but
@@ -267,6 +274,7 @@ fn longest_subgraph(
         is_equal = lp_node_equal(a, b, expr_arena);
 
         if !is_equal {
+            entirely_equal = false;
             break;
         }
 
@@ -276,7 +284,7 @@ fn longest_subgraph(
     }
     // previous node was equal
     if i > 0 {
-        Some((i - 1, prev_node_a, prev_node_b))
+        Some((prev_node_a, prev_node_b, entirely_equal))
     } else {
         None
     }
@@ -299,14 +307,27 @@ pub(crate) fn elim_cmn_subplans(
 
     // search from the leafs upwards and find the longest shared subplans
     let mut trail_ends = vec![];
+    // if i matches j
+    // we don't need to search with j as they are equal
+    // this is very important as otherwise we get quadratic behavior
+    let mut to_skip = BTreeSet::new();
 
     for i in 0..trails.len() {
+        if to_skip.contains(&i) {
+            continue;
+        }
         let trail_i = &trails[i];
 
         // we only look forwards, then we traverse all combinations
-        for trail_j in trails.iter().skip(i + 1) {
-            if let Some(res) = longest_subgraph(trail_i, trail_j, lp_arena, expr_arena) {
-                trail_ends.push(res)
+        for (j, trail_j) in trails.iter().enumerate().skip(i + 1) {
+            if let Some((a, b, all_equal)) =
+                longest_subgraph(trail_i, trail_j, lp_arena, expr_arena)
+            {
+                // then we can skip `j` as we already searched with trail `i` which is equal
+                if all_equal {
+                    to_skip.insert(j);
+                }
+                trail_ends.push((a, b))
             }
         }
     }
@@ -317,14 +338,14 @@ pub(crate) fn elim_cmn_subplans(
     let mut changed = false;
 
     let mut cache_mapping = BTreeMap::new();
+    let mut cache_counts = PlHashMap::with_capacity(trail_ends.len());
 
-    // insert cache nodes
     for combination in trail_ends.iter() {
         // both are the same, but only point to a different location
         // in our arena so we hash one and store the hash for both locations
         // this will ensure all matches have the same hash.
-        let node1 = combination.1 .0;
-        let node2 = combination.2 .0;
+        let node1 = combination.0 .0;
+        let node2 = combination.1 .0;
 
         let cache_id = match (cache_mapping.get(&node1), cache_mapping.get(&node2)) {
             (Some(h), _) => *h,
@@ -344,13 +365,35 @@ pub(crate) fn elim_cmn_subplans(
                 cache_mapping.insert(node2, cache_id);
                 cache_id
             }
-        } as usize;
+        };
+        *cache_counts.entry(cache_id).or_insert(0usize) += 1;
+    }
+
+    // insert cache nodes
+    for combination in trail_ends.iter() {
+        // both are the same, but only point to a different location
+        // in our arena so we hash one and store the hash for both locations
+        // this will ensure all matches have the same hash.
+        let node1 = combination.0 .0;
+        let node2 = combination.1 .0;
+
+        let cache_id = match (cache_mapping.get(&node1), cache_mapping.get(&node2)) {
+            // (Some(_), Some(_)) => {
+            //     continue
+            // }
+            (Some(h), _) => *h,
+            (_, Some(h)) => *h,
+            _ => {
+                unreachable!()
+            }
+        };
+        let cache_count = *cache_counts.get(&cache_id).unwrap();
 
         // reassign old nodes to another location as we are going to replace
         // them with a cache node
-        for inp_node in [combination.1, combination.2] {
+        for inp_node in [combination.0, combination.1] {
             if let ALogicalPlan::Cache { count, .. } = lp_arena.get_mut(inp_node) {
-                *count += 1;
+                *count = cache_count;
             } else {
                 let lp = lp_arena.get(inp_node).clone();
 
@@ -360,7 +403,7 @@ pub(crate) fn elim_cmn_subplans(
                     input: node,
                     id: cache_id,
                     // remove after one cache hit.
-                    count: 1,
+                    count: cache_count,
                 };
                 lp_arena.replace(inp_node, cache_lp.clone());
             };
