@@ -236,3 +236,60 @@ def test_is_nan_on_non_boolean() -> None:
         pl.Series([1, 2, 3]).fill_nan(0)
     with pytest.raises(pl.InvalidOperationError):
         pl.Series(["1", "2", "3"]).fill_nan("2")  # type: ignore[arg-type]
+
+
+def test_window_expression_different_group_length() -> None:
+    try:
+        pl.DataFrame({"groups": ["a", "a", "b", "a", "b"]}).select(
+            [pl.col("groups").apply(lambda _: pl.Series([1, 2])).over("groups")]
+        )
+    except pl.ComputeError as e:
+        msg = str(e)
+        assert (
+            "The length of the window expression did not match that of the group."
+            in msg
+        )
+        assert "Group:" in msg
+        assert "Group length:" in msg
+        assert "Output: 'shape:" in msg
+
+
+@typing.no_type_check
+def test_lazy_concat_err() -> None:
+    df1 = pl.DataFrame(
+        {
+            "foo": [1, 2],
+            "bar": [6, 7],
+            "ham": ["a", "b"],
+        }
+    )
+    df2 = pl.DataFrame(
+        {
+            "foo": [3, 4],
+            "ham": ["c", "d"],
+            "bar": [8, 9],
+        }
+    )
+
+    for how in ["horizontal", "diagonal"]:
+        with pytest.raises(
+            ValueError, match="Lazy only allows 'vertical' concat strategy."
+        ):
+            pl.concat([df1.lazy(), df2.lazy()], how=how).collect()
+
+
+def test_invalid_sort_by() -> None:
+    df = pl.DataFrame(
+        {
+            "a": ["bill", "bob", "jen", "allie", "george"],
+            "b": ["M", "M", "F", "F", "M"],
+            "c": [32, 40, 20, 19, 39],
+        }
+    )
+
+    # `select a where b order by c desc`
+    with pytest.raises(
+        pl.ComputeError,
+        match="The sortby operation produced a different length than the Series that has to be sorted.",  # noqa: E501
+    ):
+        df.select(pl.col("a").filter(pl.col("b") == "M").sort_by("c", True))
