@@ -35,19 +35,23 @@ def test_lazy() -> None:
     )
 
     # test if pl.list is available, this is `to_list` re-exported as list
-    df.groupby("a").agg(pl.list("b"))
+    eager = df.groupby("a").agg(pl.list("b"))
+    assert sorted(eager.rows()) == [(1, [1.0]), (2, [2.0]), (3, [3.0])]
 
     # profile lazyframe operation/plan
-    df.lazy().groupby("a").agg(pl.list("b")).profile()
+    lazy = df.lazy().groupby("a").agg(pl.list("b"))
+    profiling_info = lazy.profile()
     # ┌──────────────┬───────┬─────┐
     # │ node         ┆ start ┆ end │
     # │ ---          ┆ ---   ┆ --- │
     # │ str          ┆ u64   ┆ u64 │
     # ╞══════════════╪═══════╪═════╡
-    # │ optimization ┆ 0     ┆ 6   │
+    # │ optimization ┆ 0     ┆ 69  │
     # ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┤
-    # │ groupby(a)   ┆ 6     ┆ 230 │
+    # │ groupby(a)   ┆ 69    ┆ 342 │
     # └──────────────┴───────┴─────┘
+    assert len(profiling_info) == 2
+    assert profiling_info[1].columns == ["node", "start", "end"]
 
 
 def test_lazyframe_membership_operator() -> None:
@@ -69,9 +73,12 @@ def test_apply() -> None:
 
 
 def test_add_eager_column() -> None:
-    df = pl.DataFrame({"a": [1, 2, 3], "b": [1.0, 2.0, 3.0]})
-    out = df.lazy().with_column(pl.lit(pl.Series("c", [1, 2, 3]))).collect()
+    ldf = pl.DataFrame({"a": [1, 2, 3], "b": [1.0, 2.0, 3.0]}).lazy()
+    assert ldf.width == 2
+
+    out = ldf.with_column(pl.lit(pl.Series("c", [1, 2, 3]))).collect()
     assert out["c"].sum() == 6
+    assert out.width == 3
 
 
 def test_set_null() -> None:
@@ -292,27 +299,29 @@ def test_arg_sort() -> None:
 
 
 def test_window_function() -> None:
-    df = pl.DataFrame(
+    ldf = pl.DataFrame(
         {
             "A": [1, 2, 3, 4, 5],
             "fruits": ["banana", "banana", "apple", "apple", "banana"],
             "B": [5, 4, 3, 2, 1],
             "cars": ["beetle", "audi", "beetle", "beetle", "beetle"],
         }
-    )
+    ).lazy()
+    assert ldf.width == 4
 
-    q = df.lazy().with_columns(
+    q = ldf.with_columns(
         [
             pl.sum("A").over("fruits").alias("fruit_sum_A"),
             pl.first("B").over("fruits").alias("fruit_first_B"),
             pl.max("B").over("cars").alias("cars_max_B"),
         ]
     )
-    out = q.collect()
-    assert out["cars_max_B"].to_list() == [5, 4, 5, 5, 5]
+    assert q.width == 7
 
-    out = df.select([pl.first("B").over(["fruits", "cars"]).alias("B_first")])
-    assert out["B_first"].to_list() == [5, 4, 3, 3, 5]
+    assert q.collect()["cars_max_B"].to_list() == [5, 4, 5, 5, 5]
+
+    out = ldf.select([pl.first("B").over(["fruits", "cars"]).alias("B_first")])
+    assert out.collect()["B_first"].to_list() == [5, 4, 3, 3, 5]
 
 
 def test_when_then_flatten() -> None:
@@ -1201,6 +1210,44 @@ def test_is_between(fruits_cars: pl.DataFrame) -> None:
     )
 
 
+def test_is_between_data_types() -> None:
+    df = pl.DataFrame(
+        {
+            "flt": [1.4, 1.2, 2.5],
+            "int": [2, 3, 4],
+            "date": [date(2020, 1, 1), date(2020, 2, 2), date(2020, 3, 3)],
+            "datetime": [
+                datetime(2020, 1, 1, 0, 0, 0),
+                datetime(2020, 1, 1, 10, 0, 0),
+                datetime(2020, 1, 1, 12, 0, 0),
+            ],
+        }
+    )
+
+    # on purpose, for float and int, we pass in a mixture of bound data types
+    assert_series_equal(
+        df.select(pl.col("flt").is_between(1, 2.3))[:, 0],
+        pl.Series("is_between", [True, True, False]),
+    )
+    assert_series_equal(
+        df.select(pl.col("int").is_between(1.5, 4))[:, 0],
+        pl.Series("is_between", [True, True, False]),
+    )
+
+    assert_series_equal(
+        df.select(pl.col("date").is_between(date(2019, 1, 1), date(2020, 2, 5)))[:, 0],
+        pl.Series("is_between", [True, True, False]),
+    )
+    assert_series_equal(
+        df.select(
+            pl.col("datetime").is_between(
+                datetime(2020, 1, 1, 5, 0, 0), datetime(2020, 1, 1, 11, 0, 0)
+            )
+        )[:, 0],
+        pl.Series("is_between", [False, True, False]),
+    )
+
+
 def test_unique() -> None:
     df = pl.DataFrame({"a": [1, 2, 2], "b": [3, 3, 3]})
 
@@ -1236,6 +1283,23 @@ def test_max_min_multiple_columns(fruits_cars: pl.DataFrame) -> None:
     assert res.to_series(0).series_equal(pl.Series("min", [1, 2, 3, 2, 1]))
 
 
+def test_max_min_wildcard_columns(fruits_cars: pl.DataFrame) -> None:
+    res = fruits_cars.select([pl.col(pl.datatypes.Int64)]).select(pl.min(["*"]))
+    assert res.to_series(0).series_equal(pl.Series("min", [1, 2, 3, 2, 1]))
+    res = fruits_cars.select([pl.col(pl.datatypes.Int64)]).select(pl.min([pl.all()]))
+    assert res.to_series(0).series_equal(pl.Series("min", [1, 2, 3, 2, 1]))
+
+    res = fruits_cars.select([pl.col(pl.datatypes.Int64)]).select(pl.max(["*"]))
+    assert res.to_series(0).series_equal(pl.Series("max", [5, 4, 3, 4, 5]))
+    res = fruits_cars.select([pl.col(pl.datatypes.Int64)]).select(pl.max([pl.all()]))
+    assert res.to_series(0).series_equal(pl.Series("max", [5, 4, 3, 4, 5]))
+
+    res = fruits_cars.select([pl.col(pl.datatypes.Int64)]).select(
+        pl.max([pl.all(), "A", "*"])
+    )
+    assert res.to_series(0).series_equal(pl.Series("max", [5, 4, 3, 4, 5]))
+
+
 def test_head_tail(fruits_cars: pl.DataFrame) -> None:
     res_expr = fruits_cars.select([pl.head("A", 2)])
     res_series = pl.head(fruits_cars["A"], 2)
